/*
 * Utility functions for cyucon.

**** All rights reserved.
****************************************************************************
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details at www.gnu.org
****************************************************************************
 */
#include "cyucon.h"
#include "crctab.h"

#include <stdio.h> /* bufsize, perror */
#include <ctype.h> /* isspace */
#include <stdarg.h> /* va_start, va_end for log */

/*
Initialize the comm port so we can talk to the cybiko device through it.
 */
void init_comm(struct termios *pts)
{
   /* some things we want to set arbitrarily */
   pts->c_lflag &= ~ICANON; 
   pts->c_lflag &= ~(ECHO | ECHOCTL | ECHONL);
   pts->c_cflag |= HUPCL;
   pts->c_cc[VMIN] = 1;
   pts->c_cc[VTIME] = 0;
   
   /* Standard CR/LF handling: this is a dumb terminal.
    * Do no translation:
    *  no NL -> CR/NL mapping on output, and
    *  no CR -> NL mapping on input.
    */
   pts->c_oflag &= ~ONLCR;
   pts->c_iflag &= ~ICRNL;

   /* no hardware flow control by default */
   pts->c_cflag &= ~CRTSCTS; 
   pts->c_iflag &= ~(IXON | IXOFF | IXANY);
   /* set 9600 bps speed by default */
   cfsetospeed(pts, B57600);
   cfsetispeed(pts, B57600);
}

/*
Change stdin to eliminate break characters, not block on reads.
 */
void init_stdin(struct termios *sts)
{
   /* again, some arbitrary things */
   sts->c_iflag &= ~BRKINT;
   sts->c_iflag |= IGNBRK;
   sts->c_lflag &= ~ISIG;
   sts->c_cc[VMIN] = 1;
   sts->c_cc[VTIME] = 0;
   sts->c_lflag &= ~ICANON;
   /* no local echo: allow the other end to do the echoing */
   sts->c_lflag &= ~(ECHO | ECHOCTL | ECHONL);
}


/* 
Restore original terminal settings on exit 
*/
void cleanup_termios (int signal) 
{
  tcsetattr(pf, TCSANOW, &pots);
  tcsetattr(STDIN_FILENO, TCSANOW, &sots);
  exit(0);
}

/*
Compute CRC on a buffer?
 */
unsigned int crc1 (char *buf,size_t bufLen)
{
   int i;
   int crc = 0;
   size_t bufSize=bufLen;

   if(! bufSize)
     bufSize=strlen(buf);

   for (i=0; i<bufSize; i++)
      crc = (crc * 2) ^ (buf[i]*2);
   crc = (crc & 0xffff) ^ (crc >> 17);
   MaybePrintMessage(DEBUG,"crc1 for command [%.40s] is %x",buf,crc);
   return(crc);
}
/*
 * Print a comment depending on verbosity.
 */
void MaybePrintMessage(logLevels level,const char *fmt, ...)
{
  va_list args;
  
  if(level > Verbosity)
    return;

  va_start(args,fmt);
  vfprintf(stderr,fmt,args);
  va_end(args);
  fprintf(stderr,"\n");
  return;
}
/*
 * Hang out for a little while.
 */
void delay(time_t secs,time_t msecs)
{
  fd_set fsDummy;
  struct timeval delayTime;
  
  delayTime.tv_sec=secs;
  delayTime.tv_usec=msecs;

  memset(&fsDummy,0,sizeof(fsDummy));
  select(0,&fsDummy,&fsDummy,&fsDummy,&delayTime);
  
  return;
}
void removeLeadingSpaces(char *inbuf)
{
  char *workPtr=inbuf;
  
  while((*workPtr) && isspace(*workPtr))
    ++workPtr;
  strcpy(inbuf,workPtr);

}
/*
 * See "manglefilename.c" for filename mangling code.
 */
